# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of the 'aea publish' subcommand."""

import os
import shutil
from abc import ABC, abstractmethod
from contextlib import suppress
from pathlib import Path
from shutil import copyfile
from tempfile import TemporaryDirectory
from typing import cast

import click

from aea.cli.push import _save_item_locally as _push_item_locally
from aea.cli.registry.publish import publish_agent
from aea.cli.registry.push import push_item as _push_item_remote
from aea.cli.registry.settings import REGISTRY_REMOTE, REMOTE_IPFS
from aea.cli.registry.utils import get_package_meta
from aea.cli.utils.click_utils import registry_flag
from aea.cli.utils.config import (
    get_default_remote_registry,
    get_ipfs_node_multiaddr,
    validate_item_config,
)
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import check_aea_project
from aea.cli.utils.exceptions import AEAConfigException
from aea.cli.utils.package_utils import (
    try_get_item_source_path,
    try_get_item_target_path,
)
from aea.configurations.base import AgentConfig, CRUDCollection, PublicId
from aea.configurations.constants import (
    AGENT,
    AGENTS,
    CONNECTIONS,
    CONTRACTS,
    DEFAULT_AEA_CONFIG_FILE,
    DEFAULT_README_FILE,
    ITEM_TYPE_PLURAL_TO_TYPE,
    PROTOCOLS,
    SKILLS,
)
from aea.helpers.cid import to_v0, to_v1


try:
    from aea_cli_ipfs.ipfs_utils import IPFSTool  # type: ignore

    IS_IPFS_PLUGIN_INSTALLED = True
except ImportError:
    IS_IPFS_PLUGIN_INSTALLED = False

PUSH_ITEMS_FLAG = "--push-missing"


@click.command(name="publish")
@registry_flag()
@click.option(
    "--push-missing", is_flag=True, help="Push missing components to registry."
)
@click.pass_context
def publish(
    click_context: click.Context, registry: str, push_missing: bool
) -> None:  # pylint: disable=unused-argument
    """Publish the agent to the registry."""

    publish_agent_package(click_context, registry, push_missing)


@check_aea_project
def publish_agent_package(
    click_context: click.Context, registry: str, push_missing: bool
) -> None:
    """Publish an agent package to a registry."""

    ctx = cast(Context, click_context.obj)
    _validate_pkp(ctx.agent_config.private_key_paths)
    _validate_config(ctx)

    if registry == REGISTRY_REMOTE:
        if get_default_remote_registry() == REMOTE_IPFS:
            _publish_agent_ipfs(ctx, push_missing)
        else:
            _publish_agent_remote(ctx, push_missing=push_missing)
    else:
        _save_agent_locally(ctx, is_mixed=False, push_missing=push_missing)


def _validate_config(ctx: Context) -> None:
    """
    Validate agent config.

    :param ctx: Context object.
    :raises ClickException: if validation is failed.
    """
    try:
        validate_item_config(AGENT, Path(ctx.cwd))
    except AEAConfigException as e:  # pragma: no cover
        raise click.ClickException("Failed to validate agent config. {}".format(str(e)))


def _validate_pkp(private_key_paths: CRUDCollection) -> None:
    """
    Prevent to publish agents with non-empty private_key_paths.

    :param private_key_paths: private_key_paths from agent config.
    :raises ClickException: if private_key_paths is not empty.
    """
    if private_key_paths.read_all() != []:
        raise click.ClickException(
            "You are not allowed to publish agents with non-empty private_key_paths. Use the `aea remove-key` command to remove key paths from `private_key_paths: {}` in `aea-config.yaml`."
        )


class BaseRegistry(ABC):
    """Base registry class."""

    @abstractmethod
    def check_item_present(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Check item present in registry. Raise ClickException if not found.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        :return: None
        """

    @abstractmethod
    def push_item(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Push item to registry.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        :return: None
        """

    def check_item_present_and_push(
        self, item_type_plural: str, public_id: PublicId
    ) -> None:
        """
        Check item present in registry and push if needed.Raise ClickException if not found.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        :return: None
        """

        with suppress(click.ClickException):
            return self.check_item_present(item_type_plural, public_id)

        try:
            self.push_item(item_type_plural, public_id)
        except Exception as e:
            raise click.ClickException(
                f"Failed to push missing item: {item_type_plural} {public_id}: {e}"
            ) from e

        try:
            self.check_item_present(item_type_plural, public_id)
        except Exception as e:
            raise click.ClickException(
                f"Failed to find item after push: {item_type_plural} {public_id}: {e}"
            ) from e


class IPFSRegistry(BaseRegistry):
    """IPFS registry."""

    def __init__(self, ctx: Context) -> None:
        """Initialize object."""
        super().__init__()

        self.ctx = ctx
        if not IS_IPFS_PLUGIN_INSTALLED:
            raise click.ClickException(
                "Please install ipfs plugin using `pip3 install open-aea-cli-ipfs`"
            )

        self.ipfs_tool = IPFSTool(addr=get_ipfs_node_multiaddr())

    def check_item_present(self, item_type_plural: str, public_id: PublicId) -> None:
        """Check if item is pinned on the node."""
        if not self.ipfs_tool.is_a_package(to_v0(public_id.hash)):
            raise click.ClickException(
                f"Dependency {public_id} is missing from registry.\nPlease push it first and then retry or use {PUSH_ITEMS_FLAG} flag to push automatically."
            )

    def push_item(self, item_type_plural: str, public_id: PublicId) -> None:
        """Push item to an ipfs registry."""

        try:
            component_path = try_get_item_source_path(
                self.ctx.cwd, None, item_type_plural, public_id.name
            )
        except click.ClickException:
            component_path = try_get_item_source_path(
                os.path.join(self.ctx.cwd, "vendor"),
                public_id.author,
                item_type_plural,
                public_id.name,
            )

        _, package_hash, _ = self.ipfs_tool.add(component_path)
        package_hash = to_v1(package_hash)

        click.echo("Pushed missing package with:")
        click.echo(f"\tPublicId: {public_id}")
        click.echo(f"\tPackage hash: {package_hash}")


class LocalRegistry(BaseRegistry):
    """Local directory registry."""

    def __init__(self, ctx: Context):
        """Init registry."""
        self.ctx = ctx
        try:
            self.registry_path = ctx.registry_path
        except ValueError as e:  # pragma: nocover
            raise click.ClickException(str(e))

    def check_item_present(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Check item present in registry. Raise ClickException if not found.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        """
        try:
            try_get_item_source_path(
                self.registry_path, public_id.author, item_type_plural, public_id.name
            )
        except click.ClickException as e:
            raise click.ClickException(
                f"Dependency is missing. {str(e)}\nPlease push it first and then retry or use {PUSH_ITEMS_FLAG} flag to push automatically."
            )

    def push_item(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Push item to registry.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        """
        item_type = ITEM_TYPE_PLURAL_TO_TYPE[item_type_plural]
        _push_item_locally(self.ctx, item_type, public_id)


class MixedRegistry(LocalRegistry):
    """Mixed remote and local component registry."""

    def check_item_present(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Check item present in registry. Raise ClickException if not found.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        """
        item_type = ITEM_TYPE_PLURAL_TO_TYPE[item_type_plural]
        try:
            LocalRegistry.check_item_present(self, item_type_plural, public_id)
        except click.ClickException:
            click.echo(
                f"Can not find dependency locally: {item_type} {public_id}. Trying remote registry..."
            )

        try:
            RemoteRegistry(self.ctx).check_item_present(item_type_plural, public_id)
        except click.ClickException:
            raise click.ClickException(
                f"Can not find dependency locally or remotely: {item_type} {public_id}. Try to add flag `{PUSH_ITEMS_FLAG}` to push dependency package to the registry."
            )


class RemoteRegistry(BaseRegistry):
    """Remote components registry."""

    def __init__(self, ctx: Context) -> None:
        """Init registry."""
        self.ctx = ctx

    def check_item_present(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Check item present in registry.Raise ClickException if not found.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        """
        item_type = ITEM_TYPE_PLURAL_TO_TYPE[item_type_plural]
        try:
            get_package_meta(item_type, public_id)
        except click.ClickException as e:
            raise click.ClickException(
                f"Package not found in remote registry: {str(e)}. You can try to add {PUSH_ITEMS_FLAG} flag."
            )

    def push_item(self, item_type_plural: str, public_id: PublicId) -> None:
        """
        Push item to registry.

        :param item_type_plural: str, item type.
        :param public_id: PublicId of the item to check.
        """
        item_type = ITEM_TYPE_PLURAL_TO_TYPE[item_type_plural]
        _push_item_remote(self.ctx, item_type, public_id)


def _check_dependencies_in_registry(
    registry: BaseRegistry, agent_config: AgentConfig, push_missing: bool
) -> None:
    """Check all agent dependencies present in registry."""
    for item_type_plural in (PROTOCOLS, CONTRACTS, CONNECTIONS, SKILLS):
        dependencies = getattr(agent_config, item_type_plural)
        for public_id in dependencies:
            if push_missing:
                registry.check_item_present_and_push(item_type_plural, public_id)
            else:
                registry.check_item_present(item_type_plural, public_id)


def _save_agent_locally(
    ctx: Context, is_mixed: bool = False, push_missing: bool = False
) -> None:
    """
    Save agent to local packages.

    :param ctx: the context
    :param is_mixed: whether or not to fetch in mixed mode
    :param push_missing: bool. flag to push missing items
    """
    try:
        registry_path = ctx.registry_path
    except ValueError as e:  # pragma: nocover
        raise click.ClickException(str(e))

    registry = MixedRegistry(ctx) if is_mixed else LocalRegistry(ctx)

    _check_dependencies_in_registry(registry, ctx.agent_config, push_missing)

    item_type_plural = AGENTS

    target_dir = try_get_item_target_path(
        registry_path,
        ctx.agent_config.author,
        item_type_plural,
        ctx.agent_config.name,
    )
    if not os.path.exists(target_dir):
        os.makedirs(target_dir, exist_ok=True)

    source_path = os.path.join(ctx.cwd, DEFAULT_AEA_CONFIG_FILE)
    target_path = os.path.join(target_dir, DEFAULT_AEA_CONFIG_FILE)
    copyfile(source_path, target_path)
    click.echo(
        f'Agent "{ctx.agent_config.name}" successfully saved in packages folder.'
    )


def _publish_agent_remote(ctx: Context, push_missing: bool) -> None:
    """
    Push agent to remote registry.

    :param ctx: the context
    :param push_missing: bool. flag to push missing items
    """
    registry = RemoteRegistry(ctx)
    _check_dependencies_in_registry(registry, ctx.agent_config, push_missing)
    publish_agent(ctx)


def _publish_agent_ipfs(ctx: Context, push_missing: bool) -> None:
    """
    Push agent to remote registry.

    :param ctx: the context
    :param push_missing: bool. flag to push missing items
    """
    if not IS_IPFS_PLUGIN_INSTALLED:
        raise click.ClickException("Please install IPFS cli plugin.")

    registry = IPFSRegistry(ctx)
    _check_dependencies_in_registry(registry, ctx.agent_config, push_missing)

    name = ctx.agent_config.agent_name
    config_file_source_path = os.path.join(ctx.cwd, DEFAULT_AEA_CONFIG_FILE)
    readme_source_path = os.path.join(ctx.cwd, DEFAULT_README_FILE)

    with TemporaryDirectory() as temp_dir:
        package_dir = os.path.join(temp_dir, name)
        os.makedirs(package_dir)
        config_file_target_path = os.path.join(package_dir, DEFAULT_AEA_CONFIG_FILE)
        shutil.copy(config_file_source_path, config_file_target_path)
        if os.path.exists(readme_source_path):
            readme_file_target_path = os.path.join(package_dir, DEFAULT_README_FILE)
            shutil.copy(readme_source_path, readme_file_target_path)
        _, package_hash, _ = registry.ipfs_tool.add(package_dir)
        package_hash = to_v1(package_hash)

    click.echo(
        f"Successfully published agent {name} to the Registry with.\n\tPublic ID: {ctx.agent_config.public_id}\n\tPackage hash: {package_hash}"
    )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This test module contains the tests for the `aea generate-key` sub-command."""
import json
import os
import random
import shutil
import string
import tempfile
from pathlib import Path
from typing import Type

import pytest
from aea_ledger_cosmos import CosmosCrypto
from aea_ledger_ethereum import EthereumCrypto
from aea_ledger_ethereum.test_tools.constants import ETHEREUM_PRIVATE_KEY_FILE
from aea_ledger_fetchai import FetchAICrypto
from aea_ledger_fetchai.test_tools.constants import FETCHAI_PRIVATE_KEY_FILE

from aea.cli import cli
from aea.configurations.constants import MULTIKEY_FILENAME
from aea.crypto.base import Crypto as BaseCrypto
from aea.crypto.registries import crypto_registry, make_crypto
from aea.helpers.io import open_file
from aea.helpers.sym_link import cd
from aea.test_tools.test_cases import AEATestCaseEmpty

from tests.conftest import CLI_LOG_OPTION, CliRunner


class TestGenerateKey:
    """Test that the command 'aea generate-key' works as expected."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        os.chdir(cls.t)

    def test_fetchai(self, password_or_none):
        """Test that the fetch private key is created correctly."""
        args = [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier] + (
            ["--password", password_or_none] if password_or_none is not None else []
        )
        result = self.runner.invoke(cli, args)
        assert result.exit_code == 0
        assert Path(FETCHAI_PRIVATE_KEY_FILE).exists()
        make_crypto(
            FetchAICrypto.identifier,
            private_key_path=FETCHAI_PRIVATE_KEY_FILE,
            password=password_or_none,
        )

        Path(FETCHAI_PRIVATE_KEY_FILE).unlink()

    def test_ethereum(self, password_or_none):
        """Test that the fetch private key is created correctly."""
        args = [*CLI_LOG_OPTION, "generate-key", EthereumCrypto.identifier] + (
            ["--password", password_or_none] if password_or_none is not None else []
        )
        result = self.runner.invoke(cli, args)
        assert result.exit_code == 0
        assert Path(ETHEREUM_PRIVATE_KEY_FILE).exists()
        make_crypto(
            EthereumCrypto.identifier,
            private_key_path=ETHEREUM_PRIVATE_KEY_FILE,
            password=password_or_none,
        )

        Path(ETHEREUM_PRIVATE_KEY_FILE).unlink()

    def test_all(self):
        """Test that all the private keys are created correctly when running 'aea generate-key all'."""
        result = self.runner.invoke(cli, [*CLI_LOG_OPTION, "generate-key", "all"])
        assert result.exit_code == 0

        assert Path(FETCHAI_PRIVATE_KEY_FILE).exists()
        assert Path(ETHEREUM_PRIVATE_KEY_FILE).exists()
        make_crypto(FetchAICrypto.identifier, private_key_path=FETCHAI_PRIVATE_KEY_FILE)
        make_crypto(
            EthereumCrypto.identifier, private_key_path=ETHEREUM_PRIVATE_KEY_FILE
        )

        Path(FETCHAI_PRIVATE_KEY_FILE).unlink()
        Path(ETHEREUM_PRIVATE_KEY_FILE).unlink()

    def test_invalid_ledger_id(self):
        """Test that the fetch private key is created correctly."""
        args = [*CLI_LOG_OPTION, "generate-key", "ledger"]
        result = self.runner.invoke(cli, args)
        assert result.exit_code == 1
        assert "Invalid identifier provided `ledger`" in result.stdout

    def test_no_ledger_installation_found(self):
        """Test that the fetch private key is created correctly."""
        args = [*CLI_LOG_OPTION, "generate-key", "ledger"]
        specs = crypto_registry.specs.copy()
        crypto_registry.specs = {}
        try:
            result = self.runner.invoke(cli, args)
            assert result.exit_code == 1
            assert "No ledger installation found" in result.stdout
        finally:
            crypto_registry.specs = specs

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        shutil.rmtree(cls.t)


class TestGenerateKeyWhenAlreadyExists:
    """Test that the command 'aea generate-key' asks for confirmation when a key already exists."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        os.chdir(cls.t)

    def test_fetchai(self):
        """Test that the fetchai private key is overwritten or not dependending on the user input."""
        result = self.runner.invoke(
            cli, [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier]
        )
        assert result.exit_code == 0
        assert Path(FETCHAI_PRIVATE_KEY_FILE).exists()

        # This tests if the file has been created and its content is correct.
        make_crypto(FetchAICrypto.identifier, private_key_path=FETCHAI_PRIVATE_KEY_FILE)
        content = Path(FETCHAI_PRIVATE_KEY_FILE).read_bytes()

        # Saying 'no' leave the files as it is.
        result = self.runner.invoke(
            cli, [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier], input="n"
        )
        assert result.exit_code == 0
        assert Path(FETCHAI_PRIVATE_KEY_FILE).read_bytes() == content

        # Saying 'yes' overwrites the file.
        result = self.runner.invoke(
            cli, [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier], input="y"
        )
        assert result.exit_code == 0
        assert Path(FETCHAI_PRIVATE_KEY_FILE).read_bytes() != content
        make_crypto(FetchAICrypto.identifier, private_key_path=FETCHAI_PRIVATE_KEY_FILE)

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        shutil.rmtree(cls.t)


class TestGenerateKeyWithFile:
    """Test that the command 'aea generate-key' can accept a file path."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        os.chdir(cls.t)

    def test_fetchai(self):
        """Test that the fetchai private key can be deposited in a custom file."""
        test_file = "test.txt"
        result = self.runner.invoke(
            cli, [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier, test_file]
        )
        assert result.exit_code == 0
        assert Path(test_file).exists()

        # This tests if the file has been created and its content is correct.
        crypto = make_crypto(FetchAICrypto.identifier, private_key_path=test_file)
        content = Path(test_file).read_bytes()
        assert content.decode("utf-8") == crypto.private_key

    def test_all(self):
        """Test that the all command does not allow a file to be provided."""
        test_file = "test.txt"
        result = self.runner.invoke(
            cli, [*CLI_LOG_OPTION, "generate-key", "all", test_file]
        )
        assert result.exit_code == 1

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        shutil.rmtree(cls.t)


class TestGenerateKeyWithAddKeyWithoutConnection(AEATestCaseEmpty):
    """Test that the command 'aea generate-key --add-key' works as expected."""

    keys_config_path = "agent.private_key_paths"
    args = []  # type: ignore

    def test_fetchai(self):
        """Test that the fetch private key is created correctly."""

        with cd(self._get_cwd()):
            result = self.run_cli_command(
                "config", "get", self.keys_config_path, cwd=self._get_cwd()
            )
            assert result.exit_code == 0
            assert json.loads(result.stdout_bytes) == {}

            args = [*CLI_LOG_OPTION, "generate-key", FetchAICrypto.identifier]
            result = self.run_cli_command(
                *args, "--add-key", *self.args, cwd=self._get_cwd()
            )
            assert result.exit_code == 0
            assert Path(FETCHAI_PRIVATE_KEY_FILE).exists()
            make_crypto(
                FetchAICrypto.identifier,
                private_key_path=FETCHAI_PRIVATE_KEY_FILE,
                password=None,
            )

            Path(FETCHAI_PRIVATE_KEY_FILE).unlink()

            result = self.run_cli_command(
                "config", "get", self.keys_config_path, cwd=self._get_cwd()
            )
            assert result.exit_code == 0
            agent_keys = json.loads(result.stdout_bytes)
            assert agent_keys.get(FetchAICrypto.identifier) == FETCHAI_PRIVATE_KEY_FILE


class TestGenerateKeyWithAddKeyWithConnection(
    TestGenerateKeyWithAddKeyWithoutConnection
):
    """Test that the command 'aea generate-key --add-key' works as expected."""

    keys_config_path = "agent.connection_private_key_paths"
    args = ["--connection"]  # type: ignore


class TestGenerateN(AEATestCaseEmpty):
    """Test generate N keys."""

    n = 5

    @pytest.mark.parametrize(
        argnames="crypto", argvalues=(EthereumCrypto, FetchAICrypto, CosmosCrypto)
    )
    def test_generate_without_password(
        self,
        crypto: Type[BaseCrypto],
    ) -> None:
        """Test generate keys without password."""

        with tempfile.TemporaryDirectory() as temp_dir:
            outfile = Path(temp_dir, crypto.identifier)
            result = self.run_cli_command(
                "generate-key",
                crypto.identifier,
                str(outfile),
                "-n",
                f"{self.n}",
                cwd=str(self.t),
            )

            assert result.exit_code == 0
            assert outfile.exists()

            keys = json.loads(outfile.read_text())
            assert len(keys) == self.n

    def test_generate_default_file(
        self,
    ) -> None:
        """Test generate default output file."""

        outfile = Path(self.t, MULTIKEY_FILENAME)
        result = self.run_cli_command(
            "generate-key",
            EthereumCrypto.identifier,
            "-n",
            f"{self.n}",
            cwd=str(self.t),
        )

        assert result.exit_code == 0
        assert outfile.exists()

        keys = json.loads(outfile.read_text())
        assert len(keys) == self.n

    @pytest.mark.parametrize(
        argnames="crypto", argvalues=(EthereumCrypto, FetchAICrypto, CosmosCrypto)
    )
    def test_generate_with_password(
        self,
        crypto: Type[BaseCrypto],
    ) -> None:
        """Test generate keys without password."""

        password = "".join(
            [random.choice(string.ascii_letters) for _ in range(10)],  # nosec
        )

        with tempfile.TemporaryDirectory() as temp_dir:
            outfile = Path(temp_dir, crypto.identifier)
            result = self.run_cli_command(
                "generate-key",
                crypto.identifier,
                str(outfile),
                "-n",
                f"{self.n}",
                "--password",
                password,
                cwd=str(self.t),
            )

            assert result.exit_code == 0
            assert outfile.exists()

            keys = json.loads(outfile.read_text())
            assert len(keys) == self.n

            for idx, key in enumerate(keys):
                key_file = f"{crypto.identifier}_{idx}.txt"
                with open_file(key_file, "w") as fp:
                    fp.write(key["private_key"])

                crypto_obj = crypto(private_key_path=key_file, password=password)
                assert crypto_obj.address == key["address"]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains a test for aea.test_tools.docker_image."""

import platform
import subprocess  # nosec
from unittest import mock

import docker
import pytest

from aea.test_tools.docker_image import Container, DockerImage, launch_image


class HelloWorldImage(DockerImage):
    """Wrapper to Hello World Docker image."""

    @property
    def tag(self) -> str:
        """Get the image tag."""
        return "hello-world:latest"

    def create(self) -> Container:
        """Create the container."""
        return self._client.containers.run(self.tag, detach=True)

    def wait(self, max_attempts: int = 15, sleep_rate: float = 1.0) -> bool:
        """Wait until the image is up."""
        return True


class TestHelloWorldImage:
    """Test Hello World"""

    image = HelloWorldImage(mock.Mock())

    @pytest.mark.parametrize("result", [None, not None])
    @pytest.mark.skipif(platform.system() == "Darwin", reason="no docker on github CI")
    def test_docker_binary_availability(self, result) -> None:
        """Test skip based on docker binary availability"""

        with mock.patch("shutil.which", return_value=result):
            with mock.patch("pytest.skip") as m:
                self.image._check_docker_binary_available()
                assert m.call_count == int(not bool(result))

    @pytest.mark.parametrize(
        "proc_result",
        [
            subprocess.CompletedProcess("", 1),
            subprocess.CompletedProcess("", 0, stdout=b""),
            subprocess.CompletedProcess("", 0, stdout=b"Docker version 0.0.0,"),
        ],
    )
    def test_correct_docker_binary_not_available(self, proc_result) -> None:
        """Test skip when docker binary not available"""

        with mock.patch("subprocess.run", return_value=proc_result):
            with mock.patch("pytest.skip") as m:
                self.image._check_docker_binary_available()
                m.assert_called_once()

    def test_stop_if_already_running(self) -> None:
        """Test stop if already running"""

        magic_mock = mock.MagicMock()
        magic_mock.image.tags = [self.image.tag]
        ContainerCollection = docker.models.containers.ContainerCollection
        with mock.patch.object(ContainerCollection, "list", return_value=[magic_mock]):
            with mock.patch.object(magic_mock, "stop"):
                any(launch_image(self.image))
                assert magic_mock.stop.call_count == 1

    @mock.patch.object(HelloWorldImage, "wait", return_value=False)
    def test_image_wait_returns_false(self, _) -> None:
        """Test wait returns False"""

        with mock.patch("pytest.fail") as m:
            any(launch_image(self.image))
            m.assert_called_once()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Memory usage of dialogues across the time."""
import time
import uuid
from typing import List, Tuple, Union, cast

from aea_cli_benchmark.utils import get_mem_usage_in_mb  # noqa: I100

from aea.common import Address
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue

from packages.valory.protocols.http.dialogues import HttpDialogue, HttpDialogues
from packages.valory.protocols.http.message import HttpMessage


class DialogueHandler:
    """Generate messages and process with dialogues."""

    def __init__(self) -> None:
        """Set dialogues."""
        # pylint: disable=unused-argument

        def role(m: Message, addr: Address) -> Dialogue.Role:
            return HttpDialogue.Role.CLIENT

        self.addr = self.random_string
        self.dialogues = HttpDialogues(self.addr, role_from_first_message=role)

    @property
    def random_string(self) -> str:
        """Get random string on every access."""
        return uuid.uuid4().hex

    def process_message(self) -> None:
        """Process a message with dialogues."""
        message = self.create()
        dialogue = self.update(message)
        self.reply(dialogue, message)

    def update(self, message: HttpMessage) -> HttpDialogue:
        """Update dialogues with message."""
        return cast(HttpDialogue, self.dialogues.update(message))

    @staticmethod
    def reply(dialogue: HttpDialogue, message: HttpMessage) -> Message:
        """Construct and send a response for message received."""
        return dialogue.reply(
            target_message=message,
            performative=HttpMessage.Performative.RESPONSE,
            version=message.version,
            headers="",
            status_code=200,
            status_text="Success",
            body=message.body,
        )

    def create(self) -> HttpMessage:
        """Make initial http request."""
        message = HttpMessage(
            dialogue_reference=HttpDialogues.new_self_initiated_dialogue_reference(),
            performative=HttpMessage.Performative.REQUEST,
            method="get",
            url="some url",
            headers="",
            version="",
            body=b"",
        )
        message.sender = self.random_string
        message.to = self.addr
        return message


def run(messages_amount: int) -> List[Tuple[str, Union[float, int]]]:
    """Test messages generation and memory consumption with dialogues."""
    handler = DialogueHandler()
    mem_usage_on_start = get_mem_usage_in_mb()
    start_time = time.time()
    for _ in range(messages_amount):
        handler.process_message()
    mem_usage = get_mem_usage_in_mb()

    return [
        ("Mem usage(Mb)", mem_usage - mem_usage_on_start),
        ("Time (seconds)", time.time() - start_time),
    ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2023 valory
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test messages module for http protocol."""

# pylint: disable=too-many-statements,too-many-locals,no-member,too-few-public-methods,redefined-builtin
from typing import List

from aea.test_tools.test_protocol import BaseProtocolMessagesTestCase

from packages.valory.protocols.http.message import HttpMessage


class TestMessageHttp(BaseProtocolMessagesTestCase):
    """Test for the 'http' protocol message."""

    MESSAGE_CLASS = HttpMessage

    def build_messages(self) -> List[HttpMessage]:  # type: ignore[override]
        """Build the messages to be used for testing."""
        return [
            HttpMessage(
                performative=HttpMessage.Performative.REQUEST,
                method="some str",
                url="some str",
                version="some str",
                headers="some str",
                body=b"some_bytes",
            ),
            HttpMessage(
                performative=HttpMessage.Performative.RESPONSE,
                version="some str",
                status_code=12,
                status_text="some str",
                headers="some str",
                body=b"some_bytes",
            ),
        ]

    def build_inconsistent(self) -> List[HttpMessage]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""
        return [
            HttpMessage(
                performative=HttpMessage.Performative.REQUEST,
                # skip content: method
                url="some str",
                version="some str",
                headers="some str",
                body=b"some_bytes",
            ),
            HttpMessage(
                performative=HttpMessage.Performative.RESPONSE,
                # skip content: version
                status_code=12,
                status_text="some str",
                headers="some str",
                body=b"some_bytes",
            ),
        ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Base manager class."""


import json
import logging
import re
import shutil
from abc import ABC, abstractmethod
from collections import OrderedDict
from enum import Enum
from pathlib import Path
from typing import Callable, Iterator, List, Optional
from typing import OrderedDict as OrderedDictType
from typing import Tuple, cast

from aea.configurations.base import PackageConfiguration
from aea.configurations.constants import AGENT, PACKAGE_TYPE_TO_CONFIG_FILE
from aea.configurations.data_types import PackageId, PackageType, PublicId
from aea.configurations.loader import load_configuration_object
from aea.helpers.dependency_tree import DependencyTree, dump_yaml, load_yaml
from aea.helpers.fingerprint import update_fingerprint
from aea.helpers.io import open_file
from aea.helpers.ipfs.base import IPFSHashOnly


PACKAGES_FILE = "packages.json"
PACKAGE_SOURCE_RE = re.compile(r"([a-z-_0-9A-Z]+\/[a-z-_0-9A-Z]+)((:)([a-z\.0-9_-]+))?")

PackageIdToHashMapping = OrderedDictType[PackageId, str]
ConfigLoaderCallableType = Callable[[PackageType, Path], PackageConfiguration]


def load_configuration(
    package_type: PackageType, package_path: Path
) -> PackageConfiguration:
    """
    Load a configuration, knowing the type and the path to the package root.

    :param package_type: the package type.
    :param package_path: the path to the package root.
    :return: the configuration object.
    """
    configuration_obj = load_configuration_object(package_type, package_path)
    configuration_obj._directory = package_path  # pylint: disable=protected-access
    return cast(PackageConfiguration, configuration_obj)


def load_fetch_ipfs() -> Callable[[str, PublicId, str, bool], Optional[Path]]:
    """Load fetch_ipfs method."""

    try:
        from aea_cli_ipfs.registry import (  # type: ignore # pylint: disable=import-outside-toplevel
            fetch_ipfs,
        )

        return fetch_ipfs
    except ModuleNotFoundError:  # pragma: nocover  # cause obvious
        raise ModuleNotFoundError(
            "open-aea-cli-ipfs plugin is not installed; Run pip3 install open-aea-cli-ipfs"
        )


class DepedencyMismatchErrors(Enum):
    """Dependency mismatch errors."""

    HASH_NOT_FOUND: int = 1
    HASH_DOES_NOT_MATCH: int = 2


class BasePackageManager(ABC):
    """AEA package manager"""

    path: Path

    def __init__(
        self,
        path: Path,
        config_loader: ConfigLoaderCallableType = load_configuration,
        logger: Optional[logging.Logger] = None,
    ) -> None:
        """Initialize object."""

        self.path = path
        self.config_loader = config_loader
        self._packages_file = path / PACKAGES_FILE

        self._logger = logger or logging.getLogger(name="PackageManager")
        self._logger.setLevel(logging.INFO)

    def _sync(
        self,
        packages: PackageIdToHashMapping,
        update_packages: bool = False,
        update_hashes: bool = False,
    ) -> Tuple[bool, PackageIdToHashMapping, PackageIdToHashMapping]:
        """
        Sync local packages to the remote registry.

        :param packages: Packages to sync
        :param update_packages: Update packages if the calculated hash for a
                                package does not match the one in the packages.json.
        :param update_hashes: Update hashes in the packages.json if the calculated
                              hash for a package does not match the one in the
                              packages.json.
        :return: flag specifying if sync is needes, updates hashes, list of packages needs to be updates
        """

        sync_needed = False
        hash_updates = packages.copy()
        package_updates = OrderedDict()

        for package_id in packages.copy():
            package_path = self.package_path_from_package_id(package_id)
            if package_path.exists():
                package_hash = IPFSHashOnly.get(str(package_path))
                expected_hash = packages[package_id]

                if package_hash == expected_hash:
                    continue

                if update_hashes:
                    hash_updates[package_id] = package_hash
                    continue

                if update_packages:
                    sync_needed = True
                    package_updates[package_id.with_hash(expected_hash)] = expected_hash
                    self._logger.info(
                        f"Hash does not match for {package_id}, downloading package again..."
                    )
                    continue

                raise PackageHashDoesNotMatch(
                    f"Hashes for {package_id} does not match; Calculated hash: {package_hash}; Expected hash: {expected_hash}"
                )

            sync_needed = True
            self._logger.info(f"{package_id} not found locally, downloading...")
            package_id_with_hash = package_id.with_hash(packages[package_id])
            self.add_package(package_id=package_id_with_hash, with_dependencies=True)

        return sync_needed, hash_updates, package_updates

    def iter_dependency_tree(
        self,
    ) -> Iterator[PackageId]:
        """Iterate dependency tree."""
        for level in DependencyTree.generate(packages_dir=self.path):
            for package_id in level:
                yield package_id

    def check_dependencies(
        self,
        configuration: PackageConfiguration,
    ) -> List[Tuple[PackageId, DepedencyMismatchErrors]]:
        """Verify hashes for package dependecies againts the available hashes."""

        mismatches = []
        for component_id in configuration.package_dependencies:
            package_id = PackageId(
                package_type=str(component_id.component_type),
                public_id=component_id.public_id,
            )
            exppected_hash = self.get_package_hash(package_id)
            if exppected_hash is None:
                mismatches.append((package_id, DepedencyMismatchErrors.HASH_NOT_FOUND))
                continue

            if exppected_hash != package_id.package_hash:
                mismatches.append(
                    (package_id, DepedencyMismatchErrors.HASH_DOES_NOT_MATCH)
                )
        return mismatches

    def is_dependencies_hashes_match(
        self, package_id: PackageId, configuration_obj: PackageConfiguration
    ) -> bool:
        """Check dependecies hashes match and print errors"""
        dependencies_with_mismatched_hashes = self.check_dependencies(
            configuration=configuration_obj,
        )
        for dep, failure in dependencies_with_mismatched_hashes:
            if failure == DepedencyMismatchErrors.HASH_NOT_FOUND:
                self._logger.error(
                    f"Package contains a dependency that is not defined in the `packages.json`"
                    f"\n\tPackage: {package_id}\n\tDependency: {dep.without_hash()}"
                )
                continue

            if failure == DepedencyMismatchErrors.HASH_DOES_NOT_MATCH:
                self._logger.error(
                    f"Dependency check failed\nHash does not match for {dep.without_hash()} in {package_id} configuration."
                )
                continue
        return not dependencies_with_mismatched_hashes

    def _get_package_configuration(self, package_id: PackageId) -> PackageConfiguration:
        """Get package configuration by package_id."""
        package_path = self.package_path_from_package_id(package_id)
        configuration_obj = self.config_loader(
            package_id.package_type,
            package_path,
        )
        return configuration_obj

    def get_package_dependencies(self, package_id: PackageId) -> List[PackageId]:
        """Get package dependencies by package_id."""
        configuration = self._get_package_configuration(package_id)
        dependencies: List[PackageId] = []
        for component_id in configuration.package_dependencies:
            package_id = PackageId(
                package_type=str(component_id.component_type),
                public_id=component_id.public_id,
            )
            dependencies.append(package_id)
        return dependencies

    def update_public_id_hash(
        self,
        public_id_str: str,
        package_type: str,
    ) -> str:
        """Update public id hash from the latest available hashes."""

        public_id_old = PublicId.from_str(public_id_str).without_hash()
        package_id = PackageId(package_type=package_type, public_id=public_id_old)
        package_hash = self.get_package_hash(package_id=package_id)
        return str(
            PublicId.from_json(
                {
                    **public_id_old.json,
                    "package_hash": package_hash,
                }
            )
        )

    def update_dependencies(
        self,
        package_id: PackageId,
    ) -> None:
        """Update dependency hashes to latest for a package."""

        config_file = self.get_package_config_file(package_id)
        package_config, extra = load_yaml(file_path=config_file)
        for component_type in PACKAGE_TYPE_TO_CONFIG_FILE:
            if component_type == AGENT:
                continue

            components = PackageType(component_type).to_plural()
            if components in package_config:
                package_config[components] = [
                    self.update_public_id_hash(
                        public_id_str=dependency, package_type=component_type
                    )
                    for dependency in package_config.get(components, [])
                ]

        dump_yaml(
            file_path=config_file,
            data=package_config,
            extra_data=extra,
        )

    def get_package_config_file(self, package_id: PackageId) -> Path:
        """Get package config file path."""
        package_path = self.package_path_from_package_id(
            package_id=package_id,
        )
        config_file = (
            package_path / PACKAGE_TYPE_TO_CONFIG_FILE[package_id.package_type.value]
        )
        return config_file

    def update_fingerprints(
        self,
        package_id: PackageId,
    ) -> None:
        """Update fingerprints for a package."""

        package_configuration = self.config_loader(
            package_id.package_type,
            self.package_path_from_package_id(package_id=package_id),
        )

        update_fingerprint(configuration=package_configuration)

    def add_package(
        self,
        package_id: PackageId,
        with_dependencies: bool = False,
        allow_update: bool = False,
    ) -> "BasePackageManager":
        """Add package."""
        # check already added
        actual_package_id = (
            self.get_package_version_with_hash(package_id)
            if self.is_package_files_exist(package_id)
            else None
        )

        is_update_needed = bool(actual_package_id) and (
            actual_package_id != package_id
            or actual_package_id.package_hash != package_id.package_hash
        )

        if not actual_package_id:
            # no package on fs, download one
            self._logger.info(f"Adding {package_id.without_hash()}")
            self._fetch_package(package_id)
        elif not is_update_needed:
            # actual version already, nothing to do
            return self
        elif is_update_needed and not allow_update:
            raise ValueError(
                f"Required package and package in the registry does not match: {package_id} vs {actual_package_id}"
            )
        else:
            self._logger.info(f"Updating {package_id.without_hash()}")
            self._update_package(package_id)

        if with_dependencies:
            self.add_dependencies_for_package(package_id, allow_update=allow_update)

        return self

    def _update_package(self, package_id: PackageId) -> None:
        """Remove package directory from the filesystem and download new package."""
        try:
            self._remove_package_dir(package_id)
        except (OSError, PermissionError) as e:
            raise PackageUpdateError(f"Cannot update {package_id}") from e
        self._fetch_package(package_id)

    def _remove_package_dir(self, package_id: PackageId) -> None:
        """Remove package directory from the filesystem"""
        package_path = self.package_path_from_package_id(package_id=package_id)
        shutil.rmtree(str(package_path))

    def _fetch_package(self, package_id: PackageId) -> None:
        """Fetch package from ipfs to the filesystem."""
        author_repo = self.path / package_id.author
        if not author_repo.exists():
            author_repo.mkdir()
            (author_repo / "__init__.py").touch()

        package_type_collection = author_repo / package_id.package_type.to_plural()
        if not package_type_collection.exists():
            package_type_collection.mkdir()
            (package_type_collection / "__init__.py").touch()

        download_path = package_type_collection / package_id.name
        load_fetch_ipfs()(
            str(package_id.package_type),
            package_id.public_id,
            str(download_path),
            True,
        )
        self._logger.debug(f"Downloaded {package_id.without_hash()}")

    def add_dependencies_for_package(
        self, package_id: PackageId, allow_update: bool = False
    ) -> None:
        """Add dependencies for the package specified."""
        for dependency_package_id in self.get_package_dependencies(
            package_id=package_id
        ):
            self.add_package(
                dependency_package_id, with_dependencies=True, allow_update=allow_update
            )

    def get_package_version_with_hash(self, package_id: PackageId) -> PackageId:
        """Add package_id with hash for the package presents in registry."""
        config = self._get_package_configuration(package_id=package_id)
        actual_package_id = PackageId(
            package_type=config.package_id.package_type,
            public_id=PublicId(
                author=config.package_id.public_id.author,
                name=config.package_id.public_id.name,
                version=config.package_id.public_id.version,
                package_hash=self.calculate_hash_from_package_id(package_id),
            ),
        )
        return actual_package_id

    def is_package_files_exist(self, package_id: PackageId) -> bool:
        """Check package exists in the filesystem by checking it's config file exists."""
        return self.get_package_config_file(package_id).exists()

    def package_path_from_package_id(self, package_id: PackageId) -> Path:
        """Get package path from the package id."""
        return (
            self.path
            / package_id.author
            / package_id.package_type.to_plural()
            / package_id.name
        )

    def calculate_hash_from_package_id(self, package_id: PackageId) -> str:
        """Calculate package hash from package id."""

        package_path = self.package_path_from_package_id(package_id=package_id)
        return IPFSHashOnly.get(file_path=str(package_path))

    def update_package(
        self,
        package_id: PackageId,
    ) -> "BasePackageManager":
        """Update package."""
        self._update_package(package_id)
        return self

    @abstractmethod
    def register(
        self, package_path: Path, package_type: Optional[PackageType] = None
    ) -> "BasePackageManager":
        """Add package to the index."""

    @abstractmethod
    def get_package_hash(self, package_id: PackageId) -> Optional[str]:
        """Return hash for the given package id."""

    @abstractmethod
    def sync(
        self,
        dev: bool = False,
        third_party: bool = True,
        update_packages: bool = False,
        update_hashes: bool = False,
    ) -> "BasePackageManager":
        """Sync local packages to the remote registry."""

    @abstractmethod
    def update_package_hashes(
        self,
        selector_prompt: Optional[Callable[[], str]] = None,
        skip_missing: bool = False,
    ) -> "BasePackageManager":
        """Update package.json file."""

    @abstractmethod
    def verify(self) -> int:
        """Verify fingerprints and outer hash of all available packages."""

    @property
    @abstractmethod
    def json(
        self,
    ) -> OrderedDictType:
        """Json representation"""

    def dump(self, file: Optional[Path] = None) -> None:
        """Dump package data to file."""
        file = file or self._packages_file
        with open_file(file, "w+") as fp:
            json.dump(self.json, fp, indent=4)

    @classmethod
    @abstractmethod
    def from_dir(
        cls,
        packages_dir: Path,
        config_loader: ConfigLoaderCallableType = load_configuration,
    ) -> "BasePackageManager":
        """Initialize from packages directory."""


class PackageHashDoesNotMatch(Exception):
    """Package hash does not match error."""


class PackageUpdateError(Exception):
    """Package update error."""


class PackageNotValid(Exception):
    """Package not valid."""


class PackageFileNotValid(Exception):
    """Package file not valid."""


class PackagesSourceNotValid(Exception):
    """Packages source not valid."""



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Envelopes generation speed for Behaviour act test."""
from typing import Any, List, Tuple

import click
from aea_cli_benchmark.utils import (
    multi_run,
    number_of_runs_deco,
    output_format_deco,
    print_results,
    with_packages,
)


PACKAGES = [
    ("protocol", "open_aea/signing"),
    ("protocol", "fetchai/default"),
    ("protocol", "fetchai/http"),
    ("connection", "fetchai/local"),
    ("protocol", "fetchai/oef_search"),
]


@click.command(name="multiagent_message_exchange")
@click.option("--duration", default=1, help="Run time in seconds.")
@click.option(
    "--runtime_mode", default="async", help="Runtime mode: async or threaded."
)
@click.option("--runner_mode", default="async", help="Runtime mode: async or threaded.")
@click.option(
    "--start_messages", default=100, help="Amount of messages to prepopulate."
)
@click.option("--num_of_agents", default=2, help="Amount of agents to run.")
@number_of_runs_deco
@output_format_deco
def main(
    duration: int,
    runtime_mode: str,
    runner_mode: str,
    start_messages: int,
    num_of_agents: int,
    number_of_runs: int,
    output_format: str,
) -> Any:
    """Test multiagent message exchange."""
    with with_packages(PACKAGES):
        from aea_cli_benchmark.case_multiagent.case import run

        parameters = {
            "Duration(seconds)": duration,
            "Runtime mode": runtime_mode,
            "Runner mode": runner_mode,
            "Start messages": start_messages,
            "Number of agents": num_of_agents,
            "Number of runs": number_of_runs,
        }

        def result_fn() -> List[Tuple[str, Any, Any, Any]]:
            return multi_run(
                int(number_of_runs),
                run,
                (duration, runtime_mode, runner_mode, start_messages, num_of_agents),
            )

        return print_results(output_format, parameters, result_fn)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gym.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\tgym.proto\x12\x16\x61\x65\x61.fetchai.gym.v1_0_0"\x94\x07\n\nGymMessage\x12\x42\n\x03\x61\x63t\x18\x05 \x01(\x0b\x32\x33.aea.fetchai.gym.v1_0_0.GymMessage.Act_PerformativeH\x00\x12\x46\n\x05\x63lose\x18\x06 \x01(\x0b\x32\x35.aea.fetchai.gym.v1_0_0.GymMessage.Close_PerformativeH\x00\x12J\n\x07percept\x18\x07 \x01(\x0b\x32\x37.aea.fetchai.gym.v1_0_0.GymMessage.Percept_PerformativeH\x00\x12\x46\n\x05reset\x18\x08 \x01(\x0b\x32\x35.aea.fetchai.gym.v1_0_0.GymMessage.Reset_PerformativeH\x00\x12H\n\x06status\x18\t \x01(\x0b\x32\x36.aea.fetchai.gym.v1_0_0.GymMessage.Status_PerformativeH\x00\x1a\x18\n\tAnyObject\x12\x0b\n\x03\x61ny\x18\x01 \x01(\x0c\x1a\x61\n\x10\x41\x63t_Performative\x12<\n\x06\x61\x63tion\x18\x01 \x01(\x0b\x32,.aea.fetchai.gym.v1_0_0.GymMessage.AnyObject\x12\x0f\n\x07step_id\x18\x02 \x01(\x05\x1a\xc4\x01\n\x14Percept_Performative\x12\x0f\n\x07step_id\x18\x01 \x01(\x05\x12\x41\n\x0bobservation\x18\x02 \x01(\x0b\x32,.aea.fetchai.gym.v1_0_0.GymMessage.AnyObject\x12\x0e\n\x06reward\x18\x03 \x01(\x01\x12\x0c\n\x04\x64one\x18\x04 \x01(\x08\x12:\n\x04info\x18\x05 \x01(\x0b\x32,.aea.fetchai.gym.v1_0_0.GymMessage.AnyObject\x1a\x9b\x01\n\x13Status_Performative\x12T\n\x07\x63ontent\x18\x01 \x03(\x0b\x32\x43.aea.fetchai.gym.v1_0_0.GymMessage.Status_Performative.ContentEntry\x1a.\n\x0c\x43ontentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x14\n\x12Reset_Performative\x1a\x14\n\x12\x43lose_PerformativeB\x0e\n\x0cperformativeb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "gym_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _GYMMESSAGE_STATUS_PERFORMATIVE_CONTENTENTRY._options = None
    _GYMMESSAGE_STATUS_PERFORMATIVE_CONTENTENTRY._serialized_options = b"8\001"
    _globals["_GYMMESSAGE"]._serialized_start = 38
    _globals["_GYMMESSAGE"]._serialized_end = 954
    _globals["_GYMMESSAGE_ANYOBJECT"]._serialized_start = 414
    _globals["_GYMMESSAGE_ANYOBJECT"]._serialized_end = 438
    _globals["_GYMMESSAGE_ACT_PERFORMATIVE"]._serialized_start = 440
    _globals["_GYMMESSAGE_ACT_PERFORMATIVE"]._serialized_end = 537
    _globals["_GYMMESSAGE_PERCEPT_PERFORMATIVE"]._serialized_start = 540
    _globals["_GYMMESSAGE_PERCEPT_PERFORMATIVE"]._serialized_end = 736
    _globals["_GYMMESSAGE_STATUS_PERFORMATIVE"]._serialized_start = 739
    _globals["_GYMMESSAGE_STATUS_PERFORMATIVE"]._serialized_end = 894
    _globals["_GYMMESSAGE_STATUS_PERFORMATIVE_CONTENTENTRY"]._serialized_start = 848
    _globals["_GYMMESSAGE_STATUS_PERFORMATIVE_CONTENTENTRY"]._serialized_end = 894
    _globals["_GYMMESSAGE_RESET_PERFORMATIVE"]._serialized_start = 896
    _globals["_GYMMESSAGE_RESET_PERFORMATIVE"]._serialized_end = 916
    _globals["_GYMMESSAGE_CLOSE_PERFORMATIVE"]._serialized_start = 918
    _globals["_GYMMESSAGE_CLOSE_PERFORMATIVE"]._serialized_end = 938
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Helper tools for fingerprinting packages."""

import re
from pathlib import Path
from typing import Collection, Dict, Optional, Tuple

from aea.configurations.base import (
    AgentConfig,
    PackageConfiguration,
    _compute_fingerprint,
)


def _replace_fingerprint_non_invasive(
    fingerprint_dict: Dict[str, str], text: str
) -> str:
    """
    Replace the fingerprint in a configuration file (not invasive).

    We need this function because libraries like `yaml` may modify the
    content of the .yaml file when loading/dumping. Instead,
    working with the content of the file gives us finer granularity.

    :param fingerprint_dict: the fingerprint dictionary.
    :param text: the content of a configuration file.
    :return: the updated content of the configuration file.
    """

    def to_row(x: Tuple[str, str]) -> str:
        return x[0] + ": " + x[1]

    if len(fingerprint_dict) == 0:
        replacement = "\nfingerprint: {}\n"
    else:
        replacement = "\nfingerprint:\n  {}\n".format(
            "\n  ".join(map(to_row, sorted(fingerprint_dict.items())))
        )

    return re.sub(r"\nfingerprint:\W*\n(?:\W+.*\n)*", replacement, text)


def compute_fingerprint(  # pylint: disable=unsubscriptable-object
    package_path: Path,
    fingerprint_ignore_patterns: Optional[Collection[str]],
) -> Dict[str, str]:
    """
    Compute the fingerprint of a package.

    :param package_path: path to the package.
    :param fingerprint_ignore_patterns: filename patterns whose matches will be ignored.
    :return: the fingerprint
    """
    fingerprint = _compute_fingerprint(
        package_path,
        ignore_patterns=fingerprint_ignore_patterns,
    )
    return fingerprint


def update_fingerprint(configuration: PackageConfiguration) -> None:
    """
    Update the fingerprint of a package.

    :param configuration: the configuration object.
    """

    if configuration.directory is None:
        raise ValueError("configuration.directory cannot be None.")

    fingerprint = compute_fingerprint(
        configuration.directory, configuration.fingerprint_ignore_patterns
    )
    config_filepath = (
        configuration.directory / configuration.default_configuration_filename
    )
    old_content = config_filepath.read_text()
    new_content = _replace_fingerprint_non_invasive(fingerprint, old_content)
    config_filepath.write_text(new_content)


def check_fingerprint(configuration: PackageConfiguration) -> bool:
    """
    Check the fingerprint of a package, given the loaded configuration file.

    :param configuration: the configuration object.
    :return: True if the fingerprint match, False otherwise.
    """
    # we don't process agent configurations
    if isinstance(configuration, AgentConfig):
        return True

    if configuration.directory is None:
        raise ValueError("configuration.directory cannot be None.")

    expected_fingerprint = compute_fingerprint(
        configuration.directory, configuration.fingerprint_ignore_patterns
    )
    actual_fingerprint = configuration.fingerprint
    result = expected_fingerprint == actual_fingerprint
    if not result:
        print(
            "Fingerprints do not match for {} in {}".format(
                configuration.name, configuration.directory
            )
        )
    return result



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Package manager."""


import sys
from pathlib import Path
from typing import List, cast
from warnings import warn

import click

from aea.cli.utils.click_utils import PackagesSource
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import pass_ctx
from aea.configurations.constants import PACKAGES
from aea.package_manager.base import BasePackageManager, PackageFileNotValid
from aea.package_manager.v0 import PackageManagerV0
from aea.package_manager.v1 import PackageManagerV1


try:
    from aea_cli_ipfs.registry import (  # type: ignore # noqa: F401 # pylint: disable=unused-import
        fetch_ipfs,
    )

    IS_IPFS_PLUGIN_INSTALLED = True
except ModuleNotFoundError:  # pragma: nocover  # cause obvious
    IS_IPFS_PLUGIN_INSTALLED = False


class SyncTypes:  # pylint: disable=too-few-public-methods
    """Types of syncs."""

    DEV = "dev"
    THIRD_PARTY = "third_party"
    ALL = "all"


def package_type_selector_prompt() -> str:
    """Selector prompt for local package type."""
    return click.prompt(
        "Select package type",
        type=click.Choice(
            (
                PackageManagerV1.LocalPackageType.DEV.value,
                PackageManagerV1.LocalPackageType.THIRD_PARTY.value,
            )
        ),
    )


@click.group("packages")
@click.pass_context
def package_manager(
    click_context: click.Context,  # pylint: disable=unused-argument
) -> None:
    """Local package manager."""


@package_manager.command()
@pass_ctx
@click.option(
    "--update-packages",
    is_flag=True,
    help="Download packages from a remote registry so that local packages match the hashes in packages.json.",
)
@click.option(
    "--update-hashes",
    is_flag=True,
    help="Recalculate hashes in packages.json so that they match the local packages.",
)
@click.option(
    "--dev",
    "sync_type",
    flag_value=SyncTypes.DEV,
    help="To sync dev packages.",
    default=False,
)
@click.option(
    "--third-party",
    "sync_type",
    flag_value=SyncTypes.THIRD_PARTY,
    help="To sync third party packages (default).",
    default=True,
)
@click.option(
    "--all",
    "sync_type",
    flag_value=SyncTypes.ALL,
    help="To sync all available packages.",
    default=False,
)
@click.option(
    "-s",
    "--source",
    "sources",
    type=PackagesSource(),
    help="Provide source name from where hashes can be synced.",
    multiple=True,
)
def sync(
    ctx: Context,
    update_packages: bool,
    update_hashes: bool,
    sync_type: str,
    sources: List[str],
) -> None:
    """Sync packages between packages.json and a local registry."""

    if not IS_IPFS_PLUGIN_INSTALLED:
        raise click.ClickException(
            "Please install ipfs plugin using `pip3 install open-aea-cli-ipfs`"
        )

    if update_hashes and update_packages:
        raise click.ClickException(
            "You cannot use both `--update-hashes` and `--update-packages` at the same time."
        )

    packages_dir = Path(ctx.registry_path)
    try:
        manager = get_package_manager(packages_dir)
        if isinstance(manager, PackageManagerV0):
            cast(PackageManagerV0, manager).sync(
                dev=(sync_type == SyncTypes.DEV or sync_type == SyncTypes.ALL),
                third_party=(
                    sync_type == SyncTypes.THIRD_PARTY or sync_type == SyncTypes.ALL
                ),
                update_packages=update_packages,
                update_hashes=update_hashes,
            )
        else:
            cast(PackageManagerV1, manager).sync(
                dev=(sync_type == SyncTypes.DEV or sync_type == SyncTypes.ALL),
                third_party=(
                    sync_type == SyncTypes.THIRD_PARTY or sync_type == SyncTypes.ALL
                ),
                update_packages=update_packages,
                update_hashes=update_hashes,
                sources=sources,
            )
    except Exception as e:  # pylint: disable=broad-except
        raise click.ClickException(str(e)) from e


@package_manager.command(name="lock")
@click.option(
    "--check",
    is_flag=True,
    help="Check packages.json",
)
@click.option(
    "--skip-missing",
    is_flag=True,
    help="Skip packages missing from the `packages.json` file.",
)
@pass_ctx
def lock_packages(ctx: Context, check: bool, skip_missing: bool) -> None:
    """Lock packages. Updates hashes in packages.json so that they match the local packages."""

    packages_dir = Path(ctx.registry_path)

    try:
        if check:
            packages_dir = Path(ctx.registry_path)
            click.echo("Verifying packages.json")
            return_code = get_package_manager(packages_dir).verify()
            if return_code:
                click.echo("Verification failed.")
            else:
                click.echo("Verification successful")
            sys.exit(return_code)

        click.echo("Updating hashes")
        get_package_manager(packages_dir).update_package_hashes(
            selector_prompt=package_type_selector_prompt, skip_missing=skip_missing
        ).dump()

        click.echo("Done")
    except Exception as e:  # pylint: disable=broad-except
        raise click.ClickException(str(e)) from e


@package_manager.command(name="init")
def _init_packages_repo() -> None:
    """Initialize packages repository."""

    packages_dir = Path.cwd() / PACKAGES
    if packages_dir.exists():
        raise click.ClickException(
            f"Packages repository already exists @ {packages_dir}"
        )

    packages_dir.mkdir()
    PackageManagerV1(path=packages_dir).dump()
    click.echo(f"Initialized packages repository @ {packages_dir}")


def get_package_manager(package_dir: Path) -> BasePackageManager:
    """Get package manager."""

    try:
        return PackageManagerV1.from_dir(package_dir)
    except PackageFileNotValid:
        warn(
            "The provided `packages.json` still follows an older format which will be deprecated on v2.0.0",
            DeprecationWarning,
            stacklevel=2,
        )
        click.echo(
            "The provided `packages.json` still follows an older format which will be deprecated on v2.0.0"
        )
        return PackageManagerV0.from_dir(package_dir)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains Serializers that can be used for custom types."""

import copy
from typing import Any, Dict, Tuple

from google.protobuf.struct_pb2 import (  # pylint: disable=no-name-in-module
    ListValue,
    Struct,
)


class DictProtobufStructSerializer:
    """
    Serialize python dictionaries of type DictType = Dict[str, ValueType] recursively conserving their dynamic type, using google.protobuf.Struct

    ValueType = PrimitiveType | DictType | List[ValueType]]
    PrimitiveType = bool | int | float | str | bytes
    """

    NEED_PATCH = "_need_patch"

    @classmethod
    def encode(cls, dictionary: Dict[str, Any]) -> bytes:
        """
        Serialize compatible dictionary to bytes.

        Copies entire dictionary in the process.

        :param dictionary: the dictionary to serialize
        :return: serialized bytes string
        """
        if not isinstance(dictionary, dict):
            raise TypeError(  # pragma: nocover
                "dictionary must be of dict type, got type {}".format(type(dictionary))
            )
        patched_dict = copy.deepcopy(dictionary)
        cls._patch_dict(patched_dict)
        pstruct = Struct()
        pstruct.update(patched_dict)  # pylint: disable=no-member
        return pstruct.SerializeToString(deterministic=True)

    @classmethod
    def decode(cls, buffer: bytes) -> Dict[str, Any]:
        """Deserialize a compatible dictionary"""
        pstruct = Struct()
        pstruct.ParseFromString(buffer)
        dictionary = dict(pstruct)
        cls._patch_dict_restore(dictionary)
        return dictionary

    @classmethod
    def _bytes_to_str(cls, value: bytes) -> str:
        return value.decode("utf-8")

    @classmethod
    def _str_to_bytes(cls, value: str) -> bytes:
        return value.encode("utf-8")

    @classmethod
    def _patch_dict(cls, dictionnary: Dict[str, Any]) -> None:
        need_patch: Dict[str, bool] = dict()
        for key, value in dictionnary.items():
            new_value, patch_needed = cls._patch_value(value)
            if patch_needed:
                need_patch[key] = True
            dictionnary[key] = new_value

        if need_patch:
            dict_need_patch = dictionnary.get(cls.NEED_PATCH, {})
            dict_need_patch.update(need_patch)
            dictionnary[cls.NEED_PATCH] = dict_need_patch

    @classmethod
    def _patch_value(cls, value: Any) -> Tuple[Any, bool]:
        if isinstance(value, bytes):
            return cls._bytes_to_str(value), True
        if isinstance(value, int) and not isinstance(value, bool):
            return value, True
        if isinstance(value, list):
            result = []
            patched = False
            types = set()
            for v in value:
                types.add(type(v))
                v, need_patch = cls._patch_value(v)
                if need_patch or isinstance(v, dict):
                    patched = True
                result.append(v)
            if len(types) > 1:
                raise ValueError(f"Mixed data types in list are not allowed!: {value}")
            return result, patched
        if isinstance(value, dict):
            cls._patch_dict(value)
            return value, False
        if isinstance(value, tuple([bool, float, str, Struct])):
            # do nothing for supported types
            return value, False
        if value is None:
            return None, False

        raise NotImplementedError(
            "DictProtobufStructSerializer doesn't support dict value type {}".format(
                type(value)
            )
        )

    @classmethod
    def _restore_value(cls, value: Any) -> Any:
        if isinstance(value, str):
            return cls._str_to_bytes(value)

        if isinstance(value, Struct):
            if value != Struct():
                new_dict = dict(value)
                cls._patch_dict_restore(new_dict)
                return new_dict
            return {}

        if isinstance(value, float):
            return int(value)

        if isinstance(value, (list, ListValue)):
            return [cls._restore_value(v) for v in value]  # type: ignore

        raise NotImplementedError(  # pragma: nocover
            "DictProtobufStructSerializer doesn't support dict value type {}".format(
                type(value)
            )
        )

    @classmethod
    def _patch_dict_restore(cls, dictionary: Dict[str, Any]) -> None:
        # protobuf Struct doesn't recursively convert Struct to dict
        need_patch = dictionary.pop(cls.NEED_PATCH, {})
        for key, value in dictionary.items():
            # protobuf struct doesn't recursively convert Struct to dict
            if isinstance(value, Struct):
                dictionary[key] = cls._restore_value(value)

            if key in need_patch:
                dictionary[key] = cls._restore_value(value)

            elif isinstance(value, (list, ListValue)):
                # fix list of elementes not needed to be restored
                dictionary[key] = list(value)  # type: ignore



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Plug-in system for adding new CLI commands."""
import os
import sys
import traceback
from typing import Callable, Iterable, List

import click
import pkg_resources


def with_plugins(plugins: Iterable[pkg_resources.EntryPoint]) -> Callable:
    """
    A decorator to register external CLI commands to an instance of `click.Group()`.

    :param plugins: An iterable producing one `pkg_resources.EntryPoint()` per iteration.
    :return: a click.Group instance.
    """

    def decorator(group: click.Group) -> click.Group:
        if not isinstance(group, click.Group):
            raise TypeError(
                "Plugins can only be attached to an instance of click.Group()"
            )

        for entry_point in plugins or ():
            try:
                group.add_command(entry_point.load())
            except Exception:  # pylint: disable=broad-except
                # Catch this so a busted plugin doesn't take down the CLI.
                # Handled by registering a dummy command that does nothing
                # other than explain the error.
                group.add_command(BrokenCommand(entry_point.name))

        return group

    return decorator


class BrokenCommand(click.Command):
    """
    Helper click.Command in case a broken plug-in is loaded.

    Rather than completely crash the CLI when a broken plugin is loaded, this
    class provides a modified help message informing the user that the plugin is
    broken and they should contact the owner.  If the user executes the plugin
    or specifies `--help` a traceback is reported showing the exception the
    plugin loader encountered.
    """

    def __init__(self, name: str) -> None:
        """
        Define the special help messages after instantiating a `click.Command()`.

        :param name: the name of the command.
        """

        click.Command.__init__(self, name)

        util_name = os.path.basename(sys.argv and sys.argv[0] or __file__)
        self.help = (
            "\nWarning: entry point could not be loaded. Contact "
            "its author for help.\n\n\b\n" + traceback.format_exc()
        )
        self.short_help = " Warning: could not load plugin. See `%s %s --help`." % (
            util_name,
            self.name,
        )

    def invoke(self, ctx: click.Context) -> None:
        """
        Print the traceback instead of doing nothing.

        :param ctx: the click.Context object.
        """

        click.echo(self.help, color=ctx.color)
        ctx.exit(1)

    def parse_args(self, ctx: click.Context, args: List) -> List:
        """
        Parse arguments.

        :param ctx: the click.Context object.
        :param args: the raw arguments.
        :return: the arguments, parsed.
        """
        return args



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module sets up test environment for gym skill."""
# pylint: skip-file

from pathlib import Path
from typing import cast

from aea.protocols.dialogue.base import DialogueLabel, DialogueMessage
from aea.skills.tasks import TaskManager
from aea.test_tools.test_skill import BaseSkillTestCase

from packages.fetchai.protocols.gym.custom_types import AnyObject
from packages.fetchai.protocols.gym.message import GymMessage
from packages.fetchai.skills.gym.dialogues import (
    DefaultDialogues,
    GymDialogue,
    GymDialogues,
)
from packages.fetchai.skills.gym.handlers import GymHandler
from packages.fetchai.skills.gym.helpers import ProxyEnv
from packages.fetchai.skills.gym.rl_agent import GoodPriceModel, MyRLAgent, PriceBandit
from packages.fetchai.skills.gym.tasks import GymTask


PACKAGE_ROOT = Path(__file__).parent.parent


class GymTestCase(BaseSkillTestCase):
    """Sets the gym class up for testing."""

    path_to_skill = PACKAGE_ROOT

    @classmethod
    def setup_class(cls):
        """Setup the test class."""
        cls.nb_steps = 4000
        config_overrides = {
            "handlers": {"gym": {"args": {"nb_steps": cls.nb_steps}}},
        }

        super().setup_class(config_overrides=config_overrides)

        # dialogues
        cls.default_dialogues = cast(
            DefaultDialogues, cls._skill.skill_context.default_dialogues
        )
        cls.gym_dialogues = cast(GymDialogues, cls._skill.skill_context.gym_dialogues)

        # handlers
        cls.gym_handler = cast(GymHandler, cls._skill.skill_context.handlers.gym)

        # models
        cls.task = GymTask(
            skill_context=cls._skill.skill_context,
            nb_steps=cls.nb_steps,
        )

        cls.task_manager = cast(TaskManager, cls._skill.skill_context.task_manager)
        cls.task_manager.start()

        cls.logger = cls._skill.skill_context.logger

        # mocked objects
        cls.dict_str_str = {"some_key": "some_value"}
        cls.mocked_task_id = 1
        cls.content_bytes = b"some_contents"
        cls.mocked_status_content = {"reset": "success"}
        cls.mocked_action = AnyObject("some_action")
        cls.mocked_observation = AnyObject("some_observation")
        cls.mocked_info = AnyObject("some_info")
        cls.mocked_step_id = 123
        cls.mocked_reward = 3242.23423

        cls.mocked_price = 765.23
        cls.mocked_beta_a = 2.876
        cls.mocked_beta_b = 0.8
        cls.mocked_bound = 78
        cls.mocked_nb_goods = 10

        cls.dummy_gym_dialogue = GymDialogue(
            DialogueLabel(
                ("", ""),
                "some_counterparty_address",
                cls._skill.skill_context.agent_address,
            ),
            cls._skill.skill_context.agent_address,
            role=GymDialogue.Role.AGENT,
        )

        cls.price_bandit = PriceBandit(
            cls.mocked_price, cls.mocked_beta_a, cls.mocked_beta_b
        )
        cls.good_price_model = GoodPriceModel(cls.mocked_bound)
        cls.my_rl_agent = MyRLAgent(cls.mocked_nb_goods, cls.logger)
        cls.proxy_env = ProxyEnv(cls._skill.skill_context)

        # list of messages
        cls.list_of_gym_messages = (
            DialogueMessage(GymMessage.Performative.RESET, {}),
            DialogueMessage(
                GymMessage.Performative.STATUS, {"content": cls.mocked_status_content}
            ),
            DialogueMessage(
                GymMessage.Performative.ACT,
                {"action": cls.mocked_action, "step_id": cls.mocked_step_id},
            ),
            DialogueMessage(
                GymMessage.Performative.PERCEPT,
                {
                    "step_id": cls.mocked_step_id,
                    "observation": cls.mocked_observation,
                    "reward": cls.mocked_reward,
                    "done": True,
                    "info": cls.mocked_info,
                },
            ),
            DialogueMessage(GymMessage.Performative.CLOSE, {}),
        )

    def setup(self):
        """Setup"""
        self.setup_class()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2023 fetchai
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test messages module for t_protocol protocol."""

# pylint: disable=too-many-statements,too-many-locals,no-member,too-few-public-methods,redefined-builtin
from typing import List

import pytest

from aea.test_tools.test_protocol import BaseProtocolMessagesTestCase

from tests.data.packages.fetchai.protocols.t_protocol.custom_types import DataModel
from tests.data.packages.fetchai.protocols.t_protocol.message import TProtocolMessage


data_model = DataModel(
    int_field=12,
    bool_field=True,
    bytes_field=b"",
    dict_field={},
    float_field=1.0,
    set_field=set(),
    str_field="str",
    list_field=["str"],
)


class TestMessageTProtocol(BaseProtocolMessagesTestCase):
    """Test for the 't_protocol' protocol message."""

    MESSAGE_CLASS = TProtocolMessage

    def test_messages_ok(self) -> None:
        """Run messages are ok for encode and decode."""
        raise pytest.skip("https://github.com/valory-xyz/open-aea/issues/565")

    def build_messages(self) -> List[TProtocolMessage]:  # type: ignore[override]
        """Build the messages to be used for testing."""
        return [
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_CT,
                content_ct=data_model,
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PT,
                content_bytes=b"some_bytes",
                content_int=12,
                content_float=1.0,
                content_bool=True,
                content_str="some str",
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PCT,
                content_set_bytes=frozenset([b"some_bytes"]),
                content_set_int=frozenset([12]),
                content_set_float=frozenset([1.0]),
                content_set_bool=frozenset([True]),
                content_set_str=frozenset(["some str"]),
                content_list_bytes=(b"some_bytes",),
                content_list_int=(12,),
                content_list_float=(1.0,),
                content_list_bool=(True,),
                content_list_str=("some str",),
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PMT,
                content_dict_int_bytes={12: b"some_bytes"},
                content_dict_int_int={12: 12},
                content_dict_int_float={12: 1.0},
                content_dict_int_bool={12: True},
                content_dict_int_str={12: "some str"},
                content_dict_bool_bytes={True: b"some_bytes"},
                content_dict_bool_int={True: 12},
                content_dict_bool_float={True: 1.0},
                content_dict_bool_bool={True: True},
                content_dict_bool_str={True: "some str"},
                content_dict_str_bytes={"some str": b"some_bytes"},
                content_dict_str_int={"some str": 12},
                content_dict_str_float={"some str": 1.0},
                content_dict_str_bool={"some str": True},
                content_dict_str_str={"some str": "some str"},
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_MT,
                content_union_1=12,
                content_union_2=frozenset([b"some_bytes"]),
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_O,
                content_o_ct=data_model,
                content_o_bool=True,
                content_o_set_int=frozenset([12]),
                content_o_list_bytes=(b"some_bytes",),
                content_o_dict_str_int={"some str": 12},
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_EMPTY_CONTENTS,
            ),
        ]

    def build_inconsistent(self) -> List[TProtocolMessage]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""
        return [
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_CT,
                # skip content: content_ct
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PT,
                # skip content: content_bytes
                content_int=12,
                content_float=1.0,
                content_bool=True,
                content_str="some str",
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PCT,
                # skip content: content_set_bytes
                content_set_int=frozenset([12]),
                content_set_float=frozenset([1.0]),
                content_set_bool=frozenset([True]),
                content_set_str=frozenset(["some str"]),
                content_list_bytes=(b"some_bytes",),
                content_list_int=(12,),
                content_list_float=(1.0,),
                content_list_bool=(True,),
                content_list_str=("some str",),
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_PMT,
                # skip content: content_dict_int_bytes
                content_dict_int_int={12: 12},
                content_dict_int_float={12: 1.0},
                content_dict_int_bool={12: True},
                content_dict_int_str={12: "some str"},
                content_dict_bool_bytes={True: b"some_bytes"},
                content_dict_bool_int={True: 12},
                content_dict_bool_float={True: 1.0},
                content_dict_bool_bool={True: True},
                content_dict_bool_str={True: "some str"},
                content_dict_str_bytes={"some str": b"some_bytes"},
                content_dict_str_int={"some str": 12},
                content_dict_str_float={"some str": 1.0},
                content_dict_str_bool={"some str": True},
                content_dict_str_str={"some str": "some str"},
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_MT,
                content_union_2=frozenset([b"some_bytes"]),
            ),
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_O,
                # skip content: content_o_ct
                content_o_bool="str",
                content_o_set_int=frozenset([12]),
                content_o_list_bytes=(b"some_bytes",),
                content_o_dict_str_int={"some str": 12},
            ),
        ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the handler class of the http_echo skill."""
# pylint: skip-file

import json
import logging
from pathlib import Path
from typing import cast
from unittest.mock import patch

from aea.protocols.dialogue.base import DialogueMessage
from aea.test_tools.test_skill import BaseSkillTestCase

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.skills.http_echo.dialogues import HttpDialogues
from packages.fetchai.skills.http_echo.handlers import HttpHandler
from packages.valory.protocols.http.message import HttpMessage


PACKAGE_DIR = Path(__file__).parent.parent


class TestHttpHandler(BaseSkillTestCase):
    """Test HttpHandler of http_echo."""

    path_to_skill = PACKAGE_DIR

    @classmethod
    def setup_class(cls):
        """Setup the test class."""
        super().setup_class()
        cls.http_handler = cast(
            HttpHandler, cls._skill.skill_context.handlers.http_handler
        )
        cls.logger = cls._skill.skill_context.logger

        cls.http_dialogues = cast(
            HttpDialogues, cls._skill.skill_context.http_dialogues
        )

        cls.get_method = "get"
        cls.post_method = "post"
        cls.url = "some_url"
        cls.version = "some_version"
        cls.headers = "some_headers"
        cls.body = b"some_body"
        cls.sender = "fetchai/some_skill:0.1.0"
        cls.skill_id = str(cls._skill.skill_context.skill_id)

        cls.status_code = 100
        cls.status_text = "some_status_text"

        cls.content = b"some_content"
        cls.list_of_messages = (
            DialogueMessage(
                HttpMessage.Performative.REQUEST,
                {
                    "method": cls.get_method,
                    "url": cls.url,
                    "version": cls.version,
                    "headers": cls.headers,
                    "body": cls.body,
                },
            ),
        )

    def test_setup(self):
        """Test the setup method of the http_echo handler."""
        assert self.http_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the http_echo handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = self.build_incoming_message(
            message_type=HttpMessage,
            dialogue_reference=incorrect_dialogue_reference,
            performative=HttpMessage.Performative.REQUEST,
            to=self.skill_id,
            method=self.get_method,
            url=self.url,
            version=self.version,
            headers=self.headers,
            body=self.body,
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.http_handler.handle(incoming_message)

        # after
        self.assert_quantity_in_outbox(1)

        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid http message={incoming_message}, unidentified dialogue.",
        )

        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=DefaultMessage,
            performative=DefaultMessage.Performative.ERROR,
            to=incoming_message.sender,
            sender=self.skill.skill_context.agent_address,
            error_code=DefaultMessage.ErrorCode.INVALID_DIALOGUE,
            error_msg="Invalid dialogue.",
            error_data={"http_message": incoming_message.encode()},
        )
        assert has_attributes, error_str

    def test_handle_request_get(self):
        """Test the _handle_request method of the http_echo handler where method is get."""
        # setup
        incoming_message = cast(
            HttpMessage,
            self.build_incoming_message(
                message_type=HttpMessage,
                performative=HttpMessage.Performative.REQUEST,
                to=self.skill_id,
                sender=self.sender,
                method=self.get_method,
                url=self.url,
                version=self.version,
                headers=self.headers,
                body=self.body,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.http_handler.handle(incoming_message)

        # after
        self.assert_quantity_in_outbox(1)

        mock_logger.assert_any_call(
            logging.INFO,
            "received http request with method={}, url={} and body={!r}".format(
                incoming_message.method, incoming_message.url, incoming_message.body
            ),
        )

        # _handle_get
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=HttpMessage,
            performative=HttpMessage.Performative.RESPONSE,
            to=incoming_message.sender,
            sender=incoming_message.to,
            version=incoming_message.version,
            status_code=200,
            status_text="Success",
            headers=incoming_message.headers,
            body=json.dumps({"tom": {"type": "cat", "age": 10}}).encode("utf-8"),
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO,
            f"responding with: {message}",
        )

    def test_handle_request_post(self):
        """Test the _handle_request method of the http_echo handler where method is post."""
        # setup
        incoming_message = cast(
            HttpMessage,
            self.build_incoming_message(
                message_type=HttpMessage,
                performative=HttpMessage.Performative.REQUEST,
                to=self.skill_id,
                sender=self.sender,
                method=self.post_method,
                url=self.url,
                version=self.version,
                headers=self.headers,
                body=self.body,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.http_handler.handle(incoming_message)

        # after
        self.assert_quantity_in_outbox(1)

        mock_logger.assert_any_call(
            logging.INFO,
            "received http request with method={}, url={} and body={!r}".format(
                incoming_message.method, incoming_message.url, incoming_message.body
            ),
        )

        # _handle_post
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=HttpMessage,
            performative=HttpMessage.Performative.RESPONSE,
            to=incoming_message.sender,
            sender=incoming_message.to,
            version=incoming_message.version,
            status_code=200,
            status_text="Success",
            headers=incoming_message.headers,
            body=self.body,
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO,
            f"responding with: {message}",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the http_echo handler."""
        # setup
        http_dialogue = self.prepare_skill_dialogue(
            dialogues=self.http_dialogues,
            messages=self.list_of_messages[:1],
        )
        incoming_message = cast(
            HttpMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=http_dialogue,
                performative=HttpMessage.Performative.RESPONSE,
                version=self.version,
                status_code=self.status_code,
                status_text=self.status_text,
                headers=self.headers,
                body=self.body,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.http_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle http message of performative={incoming_message.performative} in dialogue={http_dialogue}.",
        )

    def test_teardown(self):
        """Test the teardown method of the http_echo handler."""
        assert self.http_handler.teardown() is None
        self.assert_quantity_in_outbox(0)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the handler classes of the gym skill."""
# pylint: skip-file

import logging
from multiprocessing.pool import ApplyResult
from typing import cast
from unittest.mock import Mock, patch

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.protocols.gym.message import GymMessage
from packages.fetchai.skills.gym.dialogues import GymDialogue
from packages.fetchai.skills.gym.tests.intermediate_class import GymTestCase


class TestGymHandler(GymTestCase):
    """Test Gym handler of gym."""

    is_agent_to_agent_messages = False

    def test__init__(self):
        """Test the __init__ method of the gym handler."""
        assert self.gym_handler._task_id is None

    def test_setup(self):
        """Test the setup method of the gym handler."""
        # operation
        with patch.object(
            self.task_manager, "enqueue_task", return_value=self.mocked_task_id
        ) as mocked_enqueue_task:
            with patch.object(self.logger, "log") as mock_logger:
                assert self.gym_handler.setup() is None

        # after
        self.assert_quantity_in_outbox(0)

        mock_logger.assert_any_call(
            logging.INFO,
            "Gym handler: setup method called.",
        )
        mocked_enqueue_task.assert_any_call(self.gym_handler.task)
        assert self.gym_handler._task_id == self.mocked_task_id

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the gym handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = self.build_incoming_message(
            message_type=GymMessage,
            dialogue_reference=incorrect_dialogue_reference,
            performative=GymMessage.Performative.RESET,
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.gym_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid gym message={incoming_message}, unidentified dialogue.",
        )

        self.assert_quantity_in_outbox(1)

        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=DefaultMessage,
            performative=DefaultMessage.Performative.ERROR,
            to=incoming_message.sender,
            sender=str(self.skill.skill_context.skill_id),
            error_code=DefaultMessage.ErrorCode.INVALID_DIALOGUE,
            error_msg="Invalid dialogue.",
            error_data={"gym_message": incoming_message.encode()},
        )
        assert has_attributes, error_str

    def test_handle_percept_i(self):
        """Test the _handle_percept method of the gym handler where active_gym_dialogue==gym_dialogeu."""
        # setup
        gym_dialogue = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:3],
            ),
        )
        incoming_message = cast(
            GymMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=gym_dialogue,
                performative=GymMessage.Performative.PERCEPT,
                step_id=self.mocked_step_id,
                observation=self.mocked_observation,
                reward=self.mocked_reward,
                done=True,
                info=self.mocked_info,
            ),
        )

        self.gym_handler.task.proxy_env._active_dialogue = gym_dialogue

        # operation
        with patch.object(
            self.gym_handler.task.proxy_env_queue,
            "put",
        ) as mocked_put:
            self.gym_handler.handle(incoming_message)

        # after
        mocked_put.assert_any_call(incoming_message)

    def test_handle_percept_ii(self):
        """Test the _handle_percept method of the gym handler where active_gym_dialogue!=gym_dialogeu."""
        # setup
        gym_dialogue = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:3],
            ),
        )
        incoming_message = cast(
            GymMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=gym_dialogue,
                performative=GymMessage.Performative.PERCEPT,
                step_id=self.mocked_step_id,
                observation=self.mocked_observation,
                reward=self.mocked_reward,
                done=True,
                info=self.mocked_info,
            ),
        )

        gym_dialogue_ii = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:1],
            ),
        )
        self.gym_handler.task.proxy_env._active_dialogue = gym_dialogue_ii

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.gym_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            "gym dialogue not active dialogue.",
        )

    def test_handle_status_i(self):
        """Test the _handle_status method of the gym handler where active_gym_dialogue==gym_dialogeu and reset == success."""
        # setup
        gym_dialogue = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:1],
            ),
        )
        incoming_message = cast(
            GymMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=gym_dialogue,
                performative=GymMessage.Performative.STATUS,
                content=self.mocked_status_content,
            ),
        )

        self.gym_handler.task.proxy_env._active_dialogue = gym_dialogue

        # operation
        with patch.object(
            self.gym_handler.task.proxy_env_queue,
            "put",
        ) as mocked_put:
            self.gym_handler.handle(incoming_message)

        # after
        mocked_put.assert_any_call(incoming_message)

    def test_handle_status_ii(self):
        """Test the _handle_status method of the gym handler where active_gym_dialogue==gym_dialogeu and reset == success."""
        # setup
        gym_dialogue = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:1],
            ),
        )
        incoming_message = cast(
            GymMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=gym_dialogue,
                performative=GymMessage.Performative.STATUS,
                content={"reset": "failure"},
            ),
        )

        gym_dialogue_ii = cast(
            GymDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.gym_dialogues,
                messages=self.list_of_gym_messages[:1],
            ),
        )
        self.gym_handler.task.proxy_env._active_dialogue = gym_dialogue_ii

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.gym_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            "gym dialogue not active dialogue.",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the gym handler."""
        # setup
        incoming_message = self.build_incoming_message(
            message_type=GymMessage,
            performative=GymMessage.Performative.RESET,
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.gym_handler.handle(incoming_message)

        # after
        gym_dialogue = cast(
            GymDialogue, self.gym_dialogues.get_dialogue(incoming_message)
        )
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle gym message of performative={incoming_message.performative} in dialogue={gym_dialogue}.",
        )

    def test_teardown(self):
        """Test the teardown method of the gym handler."""
        # setup
        self.gym_handler._task_id = self.mocked_task_id

        mock_task_result = Mock(wraps=ApplyResult)
        mock_task_result.ready.return_value = True
        mock_task_result.successful.return_value = False

        # operation
        with patch.object(
            self.gym_handler.task, "teardown"
        ) as mocked_gym_task_teardown:
            with patch.object(
                self.task_manager, "get_task_result", return_value=mock_task_result
            ) as mocked_get_result:
                with patch.object(self.logger, "log") as mock_logger:
                    assert self.gym_handler.teardown() is None

        # after
        self.assert_quantity_in_outbox(0)

        mock_logger.assert_any_call(
            logging.INFO,
            "Gym handler: teardown method called.",
        )
        mocked_gym_task_teardown.assert_called_once()
        mocked_get_result.assert_any_call(self.mocked_task_id)
        mock_logger.assert_any_call(
            logging.WARNING,
            "Task not successful!",
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fipa.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\nfipa.proto\x12\x17\x61\x65\x61.fetchai.fipa.v1_0_0"\xc5\x0c\n\x0b\x46ipaMessage\x12J\n\x06\x61\x63\x63\x65pt\x18\x05 \x01(\x0b\x32\x38.aea.fetchai.fipa.v1_0_0.FipaMessage.Accept_PerformativeH\x00\x12\\\n\x0f\x61\x63\x63\x65pt_w_inform\x18\x06 \x01(\x0b\x32\x41.aea.fetchai.fipa.v1_0_0.FipaMessage.Accept_W_Inform_PerformativeH\x00\x12\x44\n\x03\x63\x66p\x18\x07 \x01(\x0b\x32\x35.aea.fetchai.fipa.v1_0_0.FipaMessage.Cfp_PerformativeH\x00\x12L\n\x07\x64\x65\x63line\x18\x08 \x01(\x0b\x32\x39.aea.fetchai.fipa.v1_0_0.FipaMessage.Decline_PerformativeH\x00\x12\x44\n\x03\x65nd\x18\t \x01(\x0b\x32\x35.aea.fetchai.fipa.v1_0_0.FipaMessage.End_PerformativeH\x00\x12J\n\x06inform\x18\n \x01(\x0b\x32\x38.aea.fetchai.fipa.v1_0_0.FipaMessage.Inform_PerformativeH\x00\x12V\n\x0cmatch_accept\x18\x0b \x01(\x0b\x32>.aea.fetchai.fipa.v1_0_0.FipaMessage.Match_Accept_PerformativeH\x00\x12h\n\x15match_accept_w_inform\x18\x0c \x01(\x0b\x32G.aea.fetchai.fipa.v1_0_0.FipaMessage.Match_Accept_W_Inform_PerformativeH\x00\x12L\n\x07propose\x18\r \x01(\x0b\x32\x39.aea.fetchai.fipa.v1_0_0.FipaMessage.Propose_PerformativeH\x00\x1a(\n\x0b\x44\x65scription\x12\x19\n\x11\x64\x65scription_bytes\x18\x01 \x01(\x0c\x1a\x1c\n\x05Query\x12\x13\n\x0bquery_bytes\x18\x01 \x01(\x0c\x1aM\n\x10\x43\x66p_Performative\x12\x39\n\x05query\x18\x01 \x01(\x0b\x32*.aea.fetchai.fipa.v1_0_0.FipaMessage.Query\x1aZ\n\x14Propose_Performative\x12\x42\n\x08proposal\x18\x01 \x01(\x0b\x32\x30.aea.fetchai.fipa.v1_0_0.FipaMessage.Description\x1a\xa6\x01\n\x1c\x41\x63\x63\x65pt_W_Inform_Performative\x12Y\n\x04info\x18\x01 \x03(\x0b\x32K.aea.fetchai.fipa.v1_0_0.FipaMessage.Accept_W_Inform_Performative.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xb2\x01\n"Match_Accept_W_Inform_Performative\x12_\n\x04info\x18\x01 \x03(\x0b\x32Q.aea.fetchai.fipa.v1_0_0.FipaMessage.Match_Accept_W_Inform_Performative.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x94\x01\n\x13Inform_Performative\x12P\n\x04info\x18\x01 \x03(\x0b\x32\x42.aea.fetchai.fipa.v1_0_0.FipaMessage.Inform_Performative.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x15\n\x13\x41\x63\x63\x65pt_Performative\x1a\x16\n\x14\x44\x65\x63line_Performative\x1a\x1b\n\x19Match_Accept_Performative\x1a\x12\n\x10\x45nd_PerformativeB\x0e\n\x0cperformativeb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "fipa_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY._options = None
    _FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY._serialized_options = b"8\001"
    _FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY._options = None
    _FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY._serialized_options = (
        b"8\001"
    )
    _FIPAMESSAGE_INFORM_PERFORMATIVE_INFOENTRY._options = None
    _FIPAMESSAGE_INFORM_PERFORMATIVE_INFOENTRY._serialized_options = b"8\001"
    _globals["_FIPAMESSAGE"]._serialized_start = 40
    _globals["_FIPAMESSAGE"]._serialized_end = 1645
    _globals["_FIPAMESSAGE_DESCRIPTION"]._serialized_start = 791
    _globals["_FIPAMESSAGE_DESCRIPTION"]._serialized_end = 831
    _globals["_FIPAMESSAGE_QUERY"]._serialized_start = 833
    _globals["_FIPAMESSAGE_QUERY"]._serialized_end = 861
    _globals["_FIPAMESSAGE_CFP_PERFORMATIVE"]._serialized_start = 863
    _globals["_FIPAMESSAGE_CFP_PERFORMATIVE"]._serialized_end = 940
    _globals["_FIPAMESSAGE_PROPOSE_PERFORMATIVE"]._serialized_start = 942
    _globals["_FIPAMESSAGE_PROPOSE_PERFORMATIVE"]._serialized_end = 1032
    _globals["_FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE"]._serialized_start = 1035
    _globals["_FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE"]._serialized_end = 1201
    _globals[
        "_FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY"
    ]._serialized_start = 1158
    _globals[
        "_FIPAMESSAGE_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY"
    ]._serialized_end = 1201
    _globals["_FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE"]._serialized_start = 1204
    _globals["_FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE"]._serialized_end = 1382
    _globals[
        "_FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY"
    ]._serialized_start = 1158
    _globals[
        "_FIPAMESSAGE_MATCH_ACCEPT_W_INFORM_PERFORMATIVE_INFOENTRY"
    ]._serialized_end = 1201
    _globals["_FIPAMESSAGE_INFORM_PERFORMATIVE"]._serialized_start = 1385
    _globals["_FIPAMESSAGE_INFORM_PERFORMATIVE"]._serialized_end = 1533
    _globals["_FIPAMESSAGE_INFORM_PERFORMATIVE_INFOENTRY"]._serialized_start = 1158
    _globals["_FIPAMESSAGE_INFORM_PERFORMATIVE_INFOENTRY"]._serialized_end = 1201
    _globals["_FIPAMESSAGE_ACCEPT_PERFORMATIVE"]._serialized_start = 1535
    _globals["_FIPAMESSAGE_ACCEPT_PERFORMATIVE"]._serialized_end = 1556
    _globals["_FIPAMESSAGE_DECLINE_PERFORMATIVE"]._serialized_start = 1558
    _globals["_FIPAMESSAGE_DECLINE_PERFORMATIVE"]._serialized_end = 1580
    _globals["_FIPAMESSAGE_MATCH_ACCEPT_PERFORMATIVE"]._serialized_start = 1582
    _globals["_FIPAMESSAGE_MATCH_ACCEPT_PERFORMATIVE"]._serialized_end = 1609
    _globals["_FIPAMESSAGE_END_PERFORMATIVE"]._serialized_start = 1611
    _globals["_FIPAMESSAGE_END_PERFORMATIVE"]._serialized_end = 1629
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2023 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test check packages command module."""

from dataclasses import dataclass
from pathlib import Path
from typing import Any, List
from unittest import mock

import pytest

from aea.cli.check_packages import find_all_configuration_files, get_public_id_from_yaml
from aea.test_tools.test_cases import BaseAEATestCase


check_author_patch = mock.patch("aea.cli.check_packages.check_author")
check_dependencies_patch = mock.patch("aea.cli.check_packages.check_dependencies")
find_all_packages_ids_patch = mock.patch(
    "aea.cli.check_packages.find_all_packages_ids", return_value=set()
)


def _unified_yaml_load_patch(**config) -> Any:
    return mock.patch(
        "aea.cli.check_packages.unified_yaml_load",
        return_value=dict(**config),
    )


def _find_all_configuration_files_patch(config_files: List) -> Any:
    return mock.patch(
        "aea.cli.check_packages.find_all_configuration_files",
        return_value=config_files,
    )


@dataclass
class _TestPublicIdParameters:
    """Dataclass to store parameters for a public id check test."""

    side_effect: List
    exit_code: int
    message: str


class TestCheckPackagesCommand(BaseAEATestCase):
    """Test check-packages command."""

    use_packages_dir: bool = True
    test_aea_config: Path
    test_connection_config: Path

    @classmethod
    def setup_class(cls) -> None:
        """Setup class."""
        super().setup_class()

        cls.test_aea_config = (
            cls.t / "packages" / "fetchai" / "agents" / "error_test" / "aea-config.yaml"
        )
        cls.test_connection_config = (
            cls.t / "packages" / "fetchai" / "connections" / "gym" / "connection.yaml"
        )

    def test_invocation(
        self,
    ) -> None:
        """Test command invocation."""

        result = self.invoke(
            "--registry-path",
            str(self.packages_dir_path),
            "check-packages",
        )

        assert result.exit_code == 0, result.output
        assert all(
            str(file) in result.output
            for file in find_all_configuration_files(self.packages_dir_path.absolute())
        )

    def test_dependency_not_found(
        self,
    ) -> None:
        """Test"""

        with find_all_packages_ids_patch, _find_all_configuration_files_patch(
            [self.test_aea_config]
        ):
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == 1, result.output
        assert "Missing: " in result.output, result.output

    def test_check_description_failure(
        self,
    ) -> None:
        """Test description check."""

        with _unified_yaml_load_patch(
            description=""
        ), (
            check_author_patch
        ), (
            check_dependencies_patch
        ), find_all_packages_ids_patch, _find_all_configuration_files_patch(
            [
                self.test_aea_config,
            ]
        ):
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == 1, result.output
        assert "has empty description field." in result.output

    def test_check_author_failure(
        self,
    ) -> None:
        """Test `check_author` failure."""

        with _unified_yaml_load_patch(
            author="SOME_AUTHOR"
        ), find_all_packages_ids_patch, _find_all_configuration_files_patch(
            [
                self.test_aea_config,
            ]
        ):
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == 1, result.output
        assert (
            "has an unexpected author value: expected fetchai, found 'SOME_AUTHOR'"
            in result.output
        )

    def test_check_public_id_failure(
        self,
    ) -> None:
        """Test `check_public_id` failure."""

        with mock.patch(
            "re.findall", return_value=[]
        ), _find_all_configuration_files_patch(
            [self.test_connection_config]
        ), check_dependencies_patch:
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == 1, result.output
        assert "expected unique definition of PUBLIC_ID for package" in result.output
        assert "found 0" in result.output

    @pytest.mark.parametrize(
        "test_param",
        [
            _TestPublicIdParameters(
                side_effect=[
                    [(None,)],
                    [(None, None, None)],
                ],
                exit_code=1,
                message="found 'None'",
            ),
            _TestPublicIdParameters(
                side_effect=[
                    [(None,)],
                    [],
                    [(None, None, None)],
                ],
                exit_code=1,
                message="found 'None/None:None'",
            ),
            _TestPublicIdParameters(
                side_effect=[
                    [(None,)],
                    [],
                    [("fetchai", "gym", "0.19.0")],
                ],
                exit_code=0,
                message="OK!",
            ),
            _TestPublicIdParameters(
                side_effect=[
                    [(None,)],
                    [],
                    ["", ()],
                ],
                exit_code=1,
                message="found ''",
            ),
        ],
    )
    def test_check_public_id_failure_wrong_public_id(
        self, test_param: _TestPublicIdParameters
    ) -> None:
        """Test `check_public_id` failure."""

        with mock.patch(
            "re.findall",
            side_effect=test_param.side_effect,
        ), _find_all_configuration_files_patch(
            [self.test_connection_config]
        ), check_dependencies_patch:
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == test_param.exit_code, result.output
        assert test_param.message in result.output

    def test_check_pypi_dependencies_failure(
        self,
    ) -> None:
        """Test `check_pypi_dependencies`"""

        with mock.patch(
            "aea.cli.check_packages.PyPIDependenciesCheckTool.check_imports",
            return_value=(
                [
                    "dep_1",
                ],
                [],
            ),
        ), _find_all_configuration_files_patch(
            [self.test_connection_config]
        ), check_dependencies_patch:
            result = self.invoke(
                "--registry-path",
                str(self.packages_dir_path),
                "check-packages",
            )

        assert result.exit_code == 1, result.output
        assert "unresolved imports: dep_1" in result.output, result.output


def test_get_public_id_from_yaml_failure() -> None:
    """Test `get_public_id_from_yaml` failures."""

    with _unified_yaml_load_patch(author="author"):
        with pytest.raises(KeyError, match="agent_name"):
            get_public_id_from_yaml(Path("."))



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the dialogue classes of the echo skill."""
# pylint: skip-file

import inspect
import os
from pathlib import Path
from typing import cast

from aea.test_tools.test_skill import BaseSkillTestCase, COUNTERPARTY_AGENT_ADDRESS

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.skills.echo.dialogues import DefaultDialogue, DefaultDialogues


CUR_PATH = os.path.dirname(inspect.getfile(inspect.currentframe()))  # type: ignore


class TestDialogues(BaseSkillTestCase):
    """Test dialogue class of echo."""

    path_to_skill = Path(CUR_PATH, "..")

    @classmethod
    def setup_class(cls):
        """Setup the test class."""
        super().setup_class()
        cls.default_dialogues = cast(
            DefaultDialogues, cls._skill.skill_context.default_dialogues
        )

    def test_default_dialogues(self):
        """Test the DefaultDialogues class."""
        _, dialogue = self.default_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=DefaultMessage.Performative.BYTES,
            content=b"some_content",
        )
        assert dialogue.role == DefaultDialogue.Role.AGENT
        assert dialogue.self_address == self.skill.skill_context.agent_name



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains utility methods for ipfs helpers."""
import contextlib
import os
import sys
from typing import Generator


@contextlib.contextmanager
def _protobuf_python_implementation() -> Generator:
    """
    Makes a context manager to force usage of the python implementation of the protobuf modules.

    By default cpp version of the protobuf library is loaded.
    This library does not provide all the needed tools to customize fields serialization order.
    Python verrsion allows to use internal methods of the protobuf objects to serialize.
    Custom serializations is required cause ipfs uses own version to serialize data to calculate data hash.
    # noqa: DAR301
    """
    PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION = "PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION"
    # unload modules
    saved_mods = {}
    for mod_name, mod in list(sys.modules.items()):
        if mod_name.startswith("google.protobuf"):
            saved_mods[mod_name] = mod
            del sys.modules[mod_name]

    prev_os_env = os.environ.get(PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION)
    os.environ[PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION] = "python"

    yield

    if prev_os_env is None:  # pragma: nocover
        del os.environ[PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION]
    else:  # pragma: nocover
        os.environ[PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION] = prev_os_env

    for mod_name in list(sys.modules.keys()):
        if mod_name.startswith("google.protobuf"):
            del sys.modules[mod_name]

    for mod_name, mod in saved_mods.items():
        sys.modules[mod_name] = mod



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Memory usage check."""
from typing import Any, List, Tuple

import click
from aea_cli_benchmark.utils import (
    multi_run,
    number_of_runs_deco,
    output_format_deco,
    print_results,
    with_packages,
)


PACKAGES = [("protocol", "open_aea/signing")]


@click.command(name="decision_maker")
@click.option(
    "--ledger_id",
    type=click.Choice(["ethereum", "cosmos", "fetchai"]),
    default="fetchai",
    help="Ledger id",
    show_default=True,
)
@click.option(
    "--amount_of_tx", default=100, help="Amount of tx to sign", show_default=True
)
@number_of_runs_deco
@output_format_deco
def main(
    ledger_id: str, amount_of_tx: int, number_of_runs: int, output_format: str
) -> Any:
    """Check performance of decision maker on signature signing."""

    with with_packages(PACKAGES):
        from aea_cli_benchmark.case_decision_maker.case import run

        parameters = {
            "Ledger id": ledger_id,
            "Amount of txs": amount_of_tx,
            "Number of runs": number_of_runs,
        }

        def result_fn() -> List[Tuple[str, Any, Any, Any]]:
            return multi_run(
                int(number_of_runs),
                run,
                (
                    ledger_id,
                    amount_of_tx,
                ),
            )

        return print_results(output_format, parameters, result_fn)
